
class BaseContent(object):
    '''
    Base content class
    '''
    raw_content_class = None
    
    def __init__(self,content):
        '''
        Constructor
        '''
        self.content = content
        self.webscraper = self.raw_content_class(self.content)
        
    def get(self,*args,**kwargs):
        raise NotImplementedError
    
    def filter(self,*args,**kwargs):
        raise NotImplementedError


'''
Created on May 11, 2012

@author: brian
'''
from BeautifulSoup import BeautifulSoup
from scrapr import extractor

from doc_attributes import TwitterAttribute,BlogAttribute,RSSFeedAttribute

class ProviderAnalyzer(extractor.Extractor):
    
    title = extractor.DocAttribute('title',return_str=True)
    
    description = extractor.DocAttribute('meta',value_attr='content',
                                         **{'name':'description'})
    
    twitter_url = TwitterAttribute()
    
    blog = BlogAttribute() 
    
    rss_feed = RSSFeedAttribute()
    


'''
Created on Apr 18, 2012

@author: brian
'''
__all__ = ('DocAttribute','MultipleDocAttribute')

class DocAttribute(object):
    
    def __init__(self,tag,value_attr=None,return_str=None,text=None,**kwargs):
        self.tag = tag
        self.value_attr = value_attr
        self.kwargs = kwargs
        self.return_str = return_str
        self.text = text
    
    def get_value(self,content,**kwargs):
        content_find = content.get(self.tag,self.kwargs,text=self.text)
        if self.value_attr and content_find:
            return content_find.get(self.value_attr,None)
        if self.return_str and content_find:
            return content_find.string
        
        return content_find
     
    def add_content(self,content,*args,**kwargs):
        self.value = self.get_value(content)
        
class MultipleDocAttribute(DocAttribute):
    
    def __init__(self,tag,**kwargs):
        self.tag = tag
        self.kwargs = kwargs
        
    def get_multiple_value(self,content,**kwargs):
        return content.filter(self.tag,self.kwargs)
    
    def get_value(self,content,**kwargs):
        return self.get_multiple_value(content)
    


'''
Created on May 11, 2012

@author: brian
'''
from BeautifulSoup import BeautifulSoup

from base import BaseContent

class Content(BaseContent):
    raw_content_class = BeautifulSoup
    
    def filter(self,*args,**kwargs):
        return self.webscraper.findAll(*args,**kwargs)
    
    def get(self, *args, **kwargs):
        return self.webscraper.find(*args,**kwargs)


'''
Created on May 11, 2012

@author: brian
'''
import requests,re
from scrapr.content.bs3 import Content

from scrapr import extractor


class CustomAttribute(extractor.DocAttribute):
    def __init__(self):
        pass
    
    def convert_relative_url(self,url,rel_url):
        if rel_url and rel_url.startswith('/'):
            return '{0}/{1}'.format(url,rel_url[1:])
        return rel_url
    
    def add_content(self,content,**kwargs):
        url = kwargs.get('url',None)
        self.value = self.get_value(content,url)
        
    def visit_blog_page(self,content,url):
        blog = BlogAttribute()
        blog.add_content(content,url=url)
        if blog.value:
            content = Content(requests.get(blog.value).content)
            target_value = self.get_regex_values(content)
            if target_value:
                return target_value.get('href')
        return None
    
    def get_regex_values(self,content):
        for i in self.regexes:
            content_find = content.get('a',href=i)
            if content_find:
                return content_find
                
class TwitterAttribute(CustomAttribute):
    
    regexes = (
        re.compile(r'http://twitter.com/(?P<slug>[-\w]+)$'),
        re.compile(r'http://www.twitter.com/(?P<slug>[-\w]+)$'),
        re.compile(r'http://twitter.com/#!/(?P<slug>[-\w]+)$'),
        re.compile(r'http://www.twitter.com/#!/(?P<slug>[-\w]+)$'),
        re.compile(r'http://twitter.com/(?P<slug>[-\w]+)/$'),
        re.compile(r'http://www.twitter.com/(?P<slug>[-\w]+)/$'),
        re.compile(r'http://twitter.com/#!/(?P<slug>[-\w]+)/$'),
        re.compile(r'http://www.twitter.com/#!/(?P<slug>[-\w]+)/$'),
               )
    
    def get_value(self, content,url):
        twitter = content.get('a',{'class':'twitter-follow-button'})
        if twitter:
            return twitter.get('href')
        twitter = self.get_regex_values(content)
        if twitter:
            return twitter.get('href')
        return self.visit_blog_page(content, url)
    
def find_blog_a(tag):
    regex = re.compile('blog|Blog')
    content = tag.text or ''
    if regex.search(content) and tag.name == 'a':
        return True
    
class BlogAttribute(CustomAttribute):
    
    def get_value(self, content,url):
        blog_url = content.get(find_blog_a)
        if blog_url:
            return self.convert_relative_url(url, blog_url.get('href'))
        blog_url = content.get('a',href=re.compile('blog'))
        if blog_url:
            return self.convert_relative_url(url, blog_url.get('href'))
        return self.convert_relative_url(url, blog_url)
    
class RSSFeedAttribute(CustomAttribute):

    regexes = (
        re.compile(r'http://feeds.feedburner.com/(?P<slug>[-\w]+)$'),
        re.compile(r'http://feeds.feedburner.com/(?P<slug>[-\w]+)/$'),
               )        
    
    def get_rss_xml(self,content):
        return content.get('link',type=re.compile('rss'))
    
    def get_regex_values(self, content):
        target_value = CustomAttribute.get_regex_values(self, content)
        if target_value:
            return target_value
        target_value = self.get_rss_xml(content)
        if target_value:
            return target_value
        
    def get_value(self,content,url):
        rss_feed = self.get_regex_values(content)
        if rss_feed:
            return self.convert_relative_url(url, rss_feed.get('href'))
        rss_feed = self.get_rss_xml(content)
        if rss_feed:
            return self.convert_relative_url(url,rss_feed.get('href'))
        rss_feed = self.visit_blog_page(content,url)
        return self.convert_relative_url(url, rss_feed)


'''
Created on Apr 18, 2012

@author: brian
'''
from doc_attributes import *
from content.bs3 import Content

def get_declared_variables(bases, attrs):
    doc_attributes = dict()
    da_update = doc_attributes.update
    attrs_pop = attrs.pop
    for variable_name, obj in attrs.items():
        if isinstance(obj, DocAttribute):
            da_update({variable_name:attrs_pop(variable_name)})
        
    for base in bases:
        if hasattr(base, 'base_variables'):
            if len(base.base_doc_attributes) > 0:
                da_update(base.base_doc_attributes)
    return doc_attributes

class DeclarativeVariablesMetaclass(type):
    """
    Partially ripped off from Django's forms.
    http://code.djangoproject.com/browser/django/trunk/django/forms/forms.py
    """
    def __new__(cls, name, bases, attrs):
        attrs['base_doc_attributes'] = get_declared_variables(bases, attrs)
        new_class = super(DeclarativeVariablesMetaclass,
            cls).__new__(cls, name, bases, attrs)

        return new_class


class BaseExtractor(object):
    """
    Base class for all Scrapr extractor classes.
    """       
    
    def __init__(self,content,*args,**kwargs):
        self.args = args
        self.kwargs = kwargs
        self.content = self.Meta.content_class(content)
        
    def __new__(cls,content,*args, **kwargs):
        extractor = cls.new(content,*args, **kwargs)
        extractor.get_response()
        return extractor
        
    @classmethod
    def new(cls,content,*args, **kwargs):
        obj = object.__new__(cls)
        obj.__init__(content,*args, **kwargs)
        return obj
    
    def get_response(self):
        self._doc_attributes = dict()
        self.load_base_doc_attributes()
    
    def load_base_doc_attributes(self):
        for key,value in self.base_doc_attributes.items():
            value.add_content(self.content,**self.kwargs)
            setattr(self, key, value.value)
    
    class Meta(object):
        content_class = Content
        
class Extractor(BaseExtractor):
    __metaclass__ = DeclarativeVariablesMetaclass 


from setuptools import setup,find_packages
 
version = '0.1.1'
 
LONG_DESCRIPTION = """
======
Scrapr
======

Simple Web Scraping Framework
"""
 
setup(
    name='scrapr',
    version=version,
    description="""Scrapr makes it easy to setup a model for finding specific tags, links, or text on a web page.""",
    long_description=LONG_DESCRIPTION,
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Environment :: Web Environment",
    ],
    keywords='web,scraping',
    author='Brian Jinwright',
    author_email='team@ipoots.com',
    maintainer='Brian Jinwright',
    packages=find_packages(),
    url='https://bitbucket.org/brianjinwright/scrapr',
    license='MIT',
    install_requires=['beautifulsoup>=3.2.1','beautifulsoup4>=4.0.5'],
    include_package_data=True,
    zip_safe=False,
)


'''
Created on May 11, 2012

@author: brian
'''
import requests

from extractors import ProviderAnalyzer

def scrape_site(url):
    req = requests.get(url)
    pa = ProviderAnalyzer(req.content,url=url)
    return dict(title=pa.title,description=pa.description,
        twitter_url=pa.twitter_url,blog=pa.blog,rss_feed=pa.rss_feed,
        )